﻿using System.Collections.Generic;
using System.Linq;
using cAlgo.API;
using cAlgo.API.Internals;

namespace cAlgo
{
    public class SymbolRepository
    {
        private readonly Robot _robot;
        private readonly Dictionary<string, Symbol> _cache = new Dictionary<string, Symbol>(); 

        public SymbolRepository(Robot robot)
        {
            _robot = robot;
        }

        public Symbol GetSymbol(string code)
        {
            Symbol result;
            if (_cache.TryGetValue(code, out result))
                return result;

            var codesToEterate = GetSubscrings(code).Distinct().OrderByDescending(_ => _.Length).Where(_ => _.Length >= 3);
            foreach (var cutCode in codesToEterate)
            {
                result = _robot.MarketData.GetSymbol(cutCode);
                if (result != null)
                {
                    _cache[code] = result;
                    _robot.Print("Symbol {0} found", cutCode);
                    return result;
                }
            }
            _robot.Print("Cannot find symbol {0}", code);
            return null;
        }

        static IEnumerable<string> GetSubscrings(string str)
        {
            yield return str;
            if (str.Length == 1)
            {
                yield break;
            }

            var substrings = GetSubscrings(str.Substring(0, str.Length - 1))
                            .Concat(GetSubscrings(str.Substring(1, str.Length - 1)));
            foreach (var substring in substrings)
            {
                yield return substring;
            }
        }
    }
}